/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Terry Fong, Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://imtsg14.epfl.ch/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


/* LINUX is defined in the makefile */
#if    !LINUX
#error  THIS IS A LINUX VERSION 
#endif

#include <iostream.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
#include <stdio.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <sys/time.h>	
#ifdef __cplusplus
}
#endif /* __cplusplus */

#include "tlDisplay_x11.h"

// events management
bool   tlDisplay::init_global = false;
static XEvent    _tl_event_keymsg;
static bool      _tl_event_rect;
static int       _tl_last_event;
static tlDisplay *_tl_last_display = NULL;
static tlPoint   *_tl_event_mouse_down   = new tlPoint ();
static tlPoint   *_tl_event_mouse_up     = new tlPoint ();
static tlPoint   *_tl_event_mouse_double = new tlPoint ();
static tlRect    *_tl_event_mouse_rect   = new tlRect  ();

#define TIME_FOR_A_DOUBLECLICK  0.35 /* in second */

#define TL_EVENT_JIGGLE_MAX     1

/**
 ***********************************************************************
 *
 *  Check if an event has occured and is in the event queue.
 *
 *  \return
 *    TL_EVENT if an event occured,\n
 *    TL_NO_EVENT otherwise.
 *  
 *  \deprecated
 *    tl_event_query is being replaced by tlDisplay::queryEvent().
 *
 ***********************************************************************/
int 
tl_event_query () 
{
  return (tlDisplay::queryEvent());
}


int
tlDisplay::queryEvent (void)
{ 
  if (XEventsQueued(_tlDisplay_x11_theDisplay, QueuedAfterReading) )
    return(TL_EVENT);
  else 
    return(TL_NO_EVENT);
}


/**
 ***********************************************************************
 *
 *  Process TL library windows events;
 *  result is stored in TL specific data structure.
 *  This is a blocking function.
 *
 *  \return
 *    TL_EVENT status (see \ref tlDisplay.h for details)
 *
 *  \deprecated
 *    tl_event is being replaced by tlDisplay::event().
 *
 ***********************************************************************/
int 
tl_event (void)
{
  return (tlDisplay::event());
}


int
tlDisplay::event (void)
{ 
  // note: the available input events are selected in tlDisplay::create
  //       by calling XSelectInput
  int i=0;
  int event_val = TL_NO_EVENT;
  static struct  timeval lastTimeButtonRelease ;
  struct timeval currentTime ;
  float timeDiference;
  static XEvent event;

  XNextEvent (_tlDisplay_x11_theDisplay, &event);    
 
  for (i=0; i<TL_MAX_DISPLAY; i++){
    if (_tlDisplay_x11_list[i] != 0) {
      if (event.xany.window == _tlDisplay_x11_window[i]) {

	_tl_last_display = _tlDisplay_x11_ptlDisplay[i];

        // tl event manager        
        switch (event.type) {

	case KeyPress:
          if (tlVerbose) {
            printf("tlDisplay::event: window %d received KeyPress\n", i);
	  }
	  event_val = TL_EVENT_KEYDOWN;
	  _tl_last_event = TL_EVENT_KEYDOWN;
	  _tl_event_keymsg = event;
	  break;

	case KeyRelease:
          if (tlVerbose) {
            printf("tlDisplay::event: window %d received KeyRelease\n", i);
	  }
	  event_val = TL_EVENT_KEYUP;
	  _tl_last_event = TL_EVENT_KEYUP;
	  _tl_event_keymsg = event;
	  break;

        case DestroyNotify:
          if (tlVerbose) {
            printf("tlDisplay::event: window %d received DestroyNotify\n", i);
	  }         
	  event_val = TL_EVENT_EXIT;
	  _tl_last_event = TL_EVENT_EXIT;
	  break;
	  
	case ButtonPress:       
          event_val = TL_EVENT_MOUSEDOWN;
          _tl_last_event = TL_EVENT_MOUSEDOWN;
          _tl_event_mouse_down->x = (int)event.xbutton.x;
	  _tl_event_mouse_down->y = (int)event.xbutton.y;
          if (tlVerbose) {
            printf("tlDisplay::event: window %d mouse DOWN at (%d, %d)\n",
		   i, _tl_event_mouse_down->x, _tl_event_mouse_down->y);
	  }
          break;
	  
        case ButtonRelease:
	  {
	    Window rW, cW;
	    int RX, RY, WX, WY;
	    unsigned int mask;

	    if (!XQueryPointer (_tlDisplay_x11_theDisplay, 
				_tlDisplay_x11_window[i], &rW, &cW,
				&RX, &RY, &WX, &WY, &mask)) {
	      printf ("error querying pointer\n");
	      _tl_event_mouse_up->x = (int) event.xbutton.x;
	      _tl_event_mouse_up->y = (int) event.xbutton.x;
	    }
	    else {
	      _tl_event_mouse_up->x = WX;
	      _tl_event_mouse_up->y = WY;
	    }
	  }

          _tl_event_mouse_rect->set (_tl_event_mouse_down, _tl_event_mouse_up);

          if (tlVerbose) {
            printf("tlDisplay::event: window %d mouse UP at (%d, %d)\n",
		   i, _tl_event_mouse_up->x, _tl_event_mouse_up->y);
	  }
	  
	  gettimeofday (&currentTime, NULL);  
	  timeDiference = (((float) (currentTime.tv_sec - 
				     lastTimeButtonRelease.tv_sec))
			   + ((float) (currentTime.tv_usec - 
				       lastTimeButtonRelease.tv_usec))
			   / 1000000.0);   
	  
	  if (timeDiference > TIME_FOR_A_DOUBLECLICK) {
	    /* it is a single click */
	    event_val = TL_EVENT_MOUSEUP;
	    _tl_last_event = TL_EVENT_MOUSEUP;
	  } else {
	    /* it is a double click */
	    event_val = TL_EVENT_MOUSEDBLCLK;
	    _tl_last_event = TL_EVENT_MOUSEDBLCLK;
	    if (tlVerbose) {
	      printf ("tlDisplay::event: double-click\n");
	    }
	  }

	  lastTimeButtonRelease = currentTime;

          // check if we have a rect or a point
          if ((_tl_event_mouse_rect->width <= TL_EVENT_JIGGLE_MAX) && 
              (_tl_event_mouse_rect->height <= TL_EVENT_JIGGLE_MAX)) {
            _tl_event_rect = false;
	    if (tlVerbose) {
	      printf ("tlDisplay::event: point\n");
	    }
	  }
          else {
            _tl_event_rect = true;
	    event_val = TL_EVENT_MOUSERECT;
	    _tl_last_event = TL_EVENT_MOUSERECT;
	    if (tlVerbose) {
	      printf ("tlDisplay::event: mouserect\n");
	    }
	  }
          break;

	case Expose:       
          event_val = TL_NO_EVENT;
          _tl_last_event = TL_NO_EVENT;
	  _tlDisplayImagePaint (i);
          if (tlVerbose) {
	    printf ("tlDisplay::event: window %d is exposed\n", i);
	  }
          break;

	case EnterNotify:
          if (tlVerbose) {
	    printf ("tlDisplay::event: window %d has focus\n", i);
	  }
	  break;

	default:
	  if (tlVerbose) {
	    printf ("tlDisplay::event: window %d received event %d\n", 
		    i, event.type);
	  }
	  break;
        }	
      }
    }
  }
       
  return event_val;
}


/**
 ***********************************************************************
 *
 *  Return last received event. Is set to TL_NO_EVENT once called.
 *
 *  \return
 *    TL_EVENT status (see tlDisplay.h for details)
 *    TL_NO_EVENT if there is no new event
 *
 *  \deprecated
 *    tl_event_get_last is being replaced by tlDisplay::getLastEvent().
 *
 ***********************************************************************/
int 
tl_event_get_last (void)
{
  return (tlDisplay::getLastEvent());
}


int
tlDisplay::getLastEvent (void)
{
  int res;

  res = _tl_last_event;
  _tl_last_event = TL_NO_EVENT;

  return res;
}


/**
 ***********************************************************************
 *
 *  Return a pointer to the display that generated the 
 *  last processed event.
 *
 *  \return
 *    a pointer to the display that received the last event
 *    (as given by tl_event_get_last),\n
 *    NULL if no display is associated with the event.
 *
 *  \deprecated
 *    tl_event_get_display is being replaced by tlDisplay::getLastDisplay().
 *
 ***********************************************************************/
tlDisplay*
tl_event_get_display (void)
{
  return (tlDisplay::getLastDisplay());
}


tlDisplay*
tlDisplay::getLastDisplay (void)
{
  tlDisplay* res;

  if (_tl_last_display == NULL) {
    TL_ERROR_MSG("tl_event_get_display : no event has occured ! \n");    
  }

  res = _tl_last_display;

  return res;

}


/**
 ***********************************************************************
 *
 *  Get mouse down click coordonates. User can have clicked in *any* 
 *  TL window. Use tlDisplay::getLastDisplay() to retrieve the window 
 *  that generated the event.
 *
 *  \return
 *    a pointer to a \ref tlPoint object.
 *
 *  \deprecated
 *    tl_event_get_point_down is being replaced by tlDisplay::getMouseDown().
 *
 ***********************************************************************/
tlPoint*
tl_event_get_point_down (void)
{
  return (tlDisplay::getMouseDown());
}


tlPoint*
tlDisplay::getMouseDown (void)
{
  return (_tl_event_mouse_down->copy ());
}


/**
 ***********************************************************************
 *
 *  Get mouse up click coordonates. User can have clicked in *any* 
 *  TL window. Use tlDisplay::getLastDisplay() to retrieve the window 
 *  that generated the event if necessary.
 *
 *  \return
 *    a pointer to a \ref tlPoint object.
 *
 *  \deprecated
 *    tl_event_get_point_up is being replaced by tlDisplay::getMouseUp().
 *
 ***********************************************************************/
tlPoint*
tl_event_get_point_up (void)
{
  return (tlDisplay::getMouseUp());
}


tlPoint*
tlDisplay::getMouseUp (void)
{
  return (_tl_event_mouse_up->copy());
}

int
tlDisplay::getMouseUp (tlPoint *point)
{
  return _tl_event_mouse_up->copyTo (point);
}


/**
 ***********************************************************************
 *
 *  Get mouse double click coordonates. User can have clicked in *any* 
 *  TL window. Use tlDisplay::getLastDisplay() to retrieve the window 
 *  that generated the event if necessary.
 *
 *  \return
 *    a pointer to a \ref tlPoint object.
 *
 *  \deprecated
 *    tl_event_get_point_dbl is being replaced by tlDisplay::getMouseDouble().
 *
 ***********************************************************************/
tlPoint*
tl_event_get_point_dbl (void)
{
  return (tlDisplay::getMouseDouble());
}


tlPoint*
tlDisplay::getMouseDouble (void)
{
  return (_tl_event_mouse_double->copy ());
}


/**
 ***********************************************************************
 *
 *  Get mouse defined rectangle coordonates. User can have clicked 
 *  in \c any TL window. Use tlDisplay::getLastDisplay() to retrieve 
 *  the window that generated the event if necessary.
 *
 *  \return
 *    a pointer to a \ref tlPoint object.
 *
 *  \deprecated
 *    tl_event_get_rect is being replaced by tlDisplay::getMouseRect().
 *
 ***********************************************************************/
tlRect*
tl_event_get_rect (void)
{
  return (tlDisplay::getMouseRect());
}


tlRect*
tlDisplay::getMouseRect (void)
{
  return _tl_event_mouse_rect->copy ();
}


/**
 ***********************************************************************
 *
 *  Get mouse defined rectangle coordonates. User can have clicked 
 *  in \c any TL window. Use tlDisplay::getLastDisplay() to retrieve 
 *  the window that generated the event if necessary.
 *
 *  \param point    \b [out] mouse defined rectangle coordonates
 *
 *  \return
 *    a pointer to a \ref tlPoint object.
 *
 ***********************************************************************/
int
tlDisplay::getMouseRect (tlRect *rect)
{
  return _tl_event_mouse_rect->copyTo (rect);
}


/**
 ***********************************************************************
 *
 *  Get keycode for keyboard downstroke.
 *
 *  \return
 *    the character ASCII code.
 *
 *  \deprecated
 *    tl_event_get_key_down is being replaced by tlDisplay::getKeyDown().
 *
 ***********************************************************************/
char
tl_event_get_key_down (void)
{
  return (tlDisplay::getKeyDown());
}


char
tlDisplay::getKeyDown (void)
{
  char buffer[20];
  int bufsize = 20;
  KeySym keysym;
  XComposeStatus compose;
  int charcount;
  charcount = XLookupString ((XKeyEvent *) &_tl_event_keymsg, buffer,
			     bufsize, &keysym, &compose);
  return ((char) keysym);
}


/**
 ***********************************************************************
 *
 *  Get keycode for keyboard upstroke.
 *
 *  \return
 *    the character ASCII code.
 *
 *  \deprecated
 *    tl_event_get_key_up is being replaced by tlDisplay::getKeyUp().
 *
 ***********************************************************************/
char
tl_event_get_key_up (void)
{
  return (tlDisplay::getKeyUp());
}


char
tlDisplay::getKeyUp (void)
{
  char buffer[20];
  int bufsize = 20;
  KeySym keysym;
  XComposeStatus compose;
  int charcount;
  charcount = XLookupString ((XKeyEvent *) &_tl_event_keymsg, buffer,
			     bufsize, &keysym, &compose);
  return ((char) keysym);
}


/**
 ***********************************************************************
 *
 *  Warp the mouse pointer to an absolute screen position. 
 *  (x, y) = (0, 0) = top-left corner
 *
 *  \return
 *    TL_ERROR status (see tlVision.h for details)
 *
 ***********************************************************************/
int
tlDisplay::setMousePos (int x, int y)
{
  XWarpPointer(_tlDisplay_x11_theDisplay, None, 
	       DefaultRootWindow (_tlDisplay_x11_theDisplay),
	       0, 0, 0, 0, x, y);

  return (TL_NO_ERROR);
}


/**
 ***********************************************************************
 *
 *  Wait for a keypress (keyboard upstroke)
 *
 *  \return
 *    the character ASCII code.
 *
 ***********************************************************************/
char
tlDisplay::waitKey (void)
{
  while (event() != TL_EVENT_KEYUP) {};
  return (getKeyUp());
}


/**
 ***********************************************************************
 *
 *  Wait for a mouse click (button down)
 *
 *  \return
 *    the character ASCII code.
 *
 ***********************************************************************/
void
tlDisplay::waitMouse (void)
{
  while (event() != TL_EVENT_MOUSEDOWN) {};
  return;
}



/**
 ***********************************************************************
 *
 *  Initialization of global members if necessary
 *
 *  \note
 *  This architecture is required for MS Windows .NET compatibility and
 *  can be safely ignored in Linux for now.
 *
 ***********************************************************************/
void
tlDisplay::initialize (void)
{
  if (!init_global) {
    init_global = true;
  }
}



