/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlCamera.h"



///  \file tlCamera.h
///  \brief Camera base class for TLIB.

///  \class tlCamera
///  \brief Base class for camera and image sensors
/// 
///  The tlCamera base class offers an encapsulation for image sources that are in a known location 
///  in the workspace, and possibly return calibrated (undistorted) images. An important child class
///  is the tlCameraStereo base class, which provides a generic interface to stereo engines.



///  Default constructor.
///
///  \note
///  See \ref errors "error management" for details.

tlCamera::tlCamera ()
{
  Tx = Ty = Tz = 0;
  Rx = Ry = Rz = 0;
  calibrated = false;

	tl_errno = TL_NO_ERROR;
}



///  Default destructor.
///
///  \note
///  See \ref errors "error management" for details.

tlCamera::~tlCamera ()
{
	tl_errno = TL_NO_ERROR;
}



///  Set the camera position in the workspace.
/// 
///  \param Tx camera position on the X axis.
///  \param Ty camera position on the Y axis.
///  \param Tz camera position on the Z axis.
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  This assumes a right-handed, orthogonal coordonate system.

int
tlCamera::setPosition (int Tx, 
											 int Ty, 
											 int Tz)
{
  this->Tx = Tx;
  this->Ty = Ty;
  this->Tz = Tz;


  return 0;
}



///  Set the camera orientation in the workspace.
/// 
///  \param Rx camera orientation around the X axis.
///  \param Ry camera orientation around the Y axis.
///  \param Rz camera orientation around the Z axis.
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  This assumes a right-handed, orthogonal coordonate system.

int
tlCamera::setOrientation (int Rx, 
													int Ry, 
													int Rz)
{
  this->Rx = Rx;
  this->Ry = Ry;
  this->Rz = Rz;


  return 0;
}



///  \fn tlCamera::tlCamera()
///  Default constructor.



///  \fn tlCamera::getTx ()
///  Get the position of the camera on the X axis of the workspace.
///
///  \return
///  The camera position in X.



///  \fn tlCamera::getTy ()
///  Get the position of the camera on the Y axis of the workspace.
///
///  \return
///  The camera position in Y.



///  \fn tlCamera::getTz ()
///  Get the position of the camera on the Z axis of the workspace.
///
///  \return
///  The camera position in Z.



///  \fn tlCamera::getRx ()
///  Get the orientation of the camera around the X axis of the workspace.
///
///  \return
///  The camera orientation around X.



///  \fn tlCamera::getRy ()
///  Get the orientation of the camera around the X axis of the workspace.
///
///  \return
///  The camera orientation around Y.



///  \fn tlCamera::getRz ()
///  Get the orientation of the camera around the X axis of the workspace.
///
///  \return
///  The camera orientation around Z.



/// \fn tlCamera::isCalibrated ()
///  Check whether the camera returns calibrated (undistorted) images or not.
///
///  \return
///  \e True if camera is calibrated,\n
///  \e False otherwise.
