/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


// SVS for Linux broken for now (has to do with gcc version)
#ifndef LINUX


#include "tlCameraSVSHead.h"

#define WAIT_GRAB 400



///  \file tlCameraSVSHead.h
///  \brief \ref svsDoc "VidereDesign" MEGA-DCS wrapper.

///  \class tlCameraSVSHead
///  VidereDesign stereo heads wrapper for TLIB.



///  Default constructor. Uses the default configuration file defined
///  in TL_CAMERA_SVS_FILENAME.
///
///  \note
///  See \ref errors "error management" for details.

tlCameraSVSHead::tlCameraSVSHead (int fps) : tlCameraSVSBase ()
{
  open (fps);
}



///  Specifies which configuration file to use.
///
///  \note
///  See \ref errors "error management" for details.

tlCameraSVSHead::tlCameraSVSHead (char *filename,
                                  int  fps) : tlCameraSVSBase ()
{
  open (filename, fps);
}



///  Default destructor.
///
///  \note
///  See \ref errors "error management" for details.

tlCameraSVSHead::~tlCameraSVSHead () 
{
  close ();
}



///  \fn tlCameraSVSHead::getGain()
///  Get the gain of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::getExposure()
///  Get the exposure of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::getContrast()
///  Get the contrast of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::getBrightness()
///  Get the brightness of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::getSaturation()
///  Get the saturation of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::getRedGain()
///  Get the gain of red channel of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::getBlueGain()
///  Get the gain of blue channel of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::getSharpness()
///  Get the sharpness of the SVS stereo head digitizer.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::isAutoGain()
///  Get the status of the automatic gain of the SVS stereo head digitizer.
///
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::isAutoWhite()
///  Get the status of the automatic white balance of the SVS stereo head digitizer.
///
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  \fn tlCameraSVSHead::isAutoBrightness()
///  Get the status of the automatic brightness of the SVS stereo head digitizer.
///
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.



///  Set the gain of the SVS stereo head digitizer.
///  
///  \param gain      new gain value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setGain (int gain)
{
  videoObject->gain = gain;
  videoObject->SetDigitization ();


  return 0;
}



///  Set the exposure of the SVS stereo head digitizer.
///  
///  \param exposure    new exposure value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setExposure (int exposure)
{
  videoObject->exposure = exposure;
  videoObject->SetDigitization ();


  return 0;
}



///  Set the contrast of the SVS stereo head digitizer.
///
///  \param contrast    new contrast value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setContrast (int contrast)
{
  videoObject->contrast = contrast;
  videoObject->SetDigitization ();


  return 0;
}



///  Set the brightness of the SVS stereo head digitizer.
///
///  \param brightness    new brightness value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setBrightness (int brightness)
{
  videoObject->brightness = brightness;
  videoObject->SetDigitization ();


  return 0;
}



///  Set the saturation of the SVS stereo head digitizer.
///
///  \param saturation    new saturation value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setSaturation (int saturation)
{
  videoObject->saturation = saturation;
  videoObject->SetDigitization ();


  return 0;
}



///  Set the gain of the red channel of the SVS stereo head digitizer.
///
///  \param red    new red gain value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setRedGain (int red)
{
  videoObject->red = red;
  videoObject->SetDigitization ();


  return 0;
}



///  Set the gain of the blue channel of the SVS stereo head digitizer.
///
///  \param blue    new blue gain value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setBlueGain (int blue)
{
  videoObject->blue = blue;
  videoObject->SetDigitization ();
  return 0;
}



///  Set the sharpness of the SVS stereo head digitizer.
///
///  \param sharpness    new sharpness value [%]
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setSharpness (int sharpness)
{
  videoObject->sharpness = sharpness;
  videoObject->SetDigitization ();


  return 0;
}



///  Enable/disable the automatic gain of the SVS stereo head digitizer.
///
///  \param val     true to enable automatic gain, false to disable it.
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setAutoGain (bool val)
{
  videoObject->autogain = val;
  videoObject->SetDigitization ();


  return 0;
}



///  Enable/disable the automatic white balance of the SVS stereo head digitizer.
///
///  \param val     true to enable automatic white balance, false to disable it.
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setAutoWhite (bool val)
{
  videoObject->autowhite = val;
  videoObject->SetDigitization ();


  return 0;
}



///  Enable/disable the automatic brightness of the SVS stereo head digitizer.
///
///  \param val     true to enable automatic brightness, false to disable it.
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///  
///  \sa
///  Read the \ref svsDoc "SVS documentation" for more information.

int         
tlCameraSVSHead::setAutoBrightness (bool val)
{
  videoObject->autobrightness = val;
  videoObject->SetDigitization ();


  return 0;
}




///  Open the SVS digitizer using the default configuration file.
///
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///
///  \note
///  The default configuration file is defined in TL_CAMERA_SVS_FILENAME.

int
tlCameraSVSHead::open (int fps)
{
  return open (TL_CAMERA_SVS_FILENAME, fps);
}




///  Open the SVS digitizer using the given configuration file.
///
///  \param filename    name of the configuration file
///  \param fps         image acquisition rate (frames per second) to use
///
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
///
///  \note
///  See http://www.videredesign.com/vision/svs_xp2_workaround.htm for 
///  more information on the correct fps value on Windows XP SP2.

int
tlCameraSVSHead::open (char *filename,
                       int  fps)
{
  // make a video source object, using the loaded framegrabber interface
  videoObject = getVideoObject ();
  if (!videoObject) {
    TL_ERRNO_MSG ("tlCameraSVSHead::open()", TL_ERROR_RESOURCE_NOT_AVAILABLE);
    return -1;    
  }

  // try and read in rectification parameters, otherwise use defaults
  videoObject->ReadParams(filename);

  // change fps if necessary
  if (fps > 0) videoObject->SetRate (fps);

  // open the video source
  if (!videoObject->Open ()) { 
    TL_ERRNO_MSG ("tlCameraSVSHead::open()", TL_ERROR_RESOURCE_NOT_AVAILABLE);
    return -1;
  }

  // start up the video stream
  if (!videoObject->Start()) {
    TL_ERRNO_MSG ("tlCameraSVSHead::open()", TL_ERROR_CONFIG_FAILED);
    return -1;
  }

  // enable on-chip stereo processing if available
  stoc = false;
  if (videoObject->is_proc_capable) {
    videoObject->SetProcMode (PROC_MODE_DISPARITY);
    stoc = true;
  }

  // members init
  this->width  = videoObject->GetIP()->width;
  this->height = videoObject->GetIP()->height;

  // assign image source
  sourceObject = videoObject;

  // get pointer to stereo images structure
  delete imageObject;
  imageObject = videoObject->GetImage (WAIT_GRAB);
  sp          = imageObject->GetSP();


  return 0;
}



///  Close the SVS digitizer.
///
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlCameraSVSHead::close ()
{  
  // imageObject is already deallocated by closing videoObject
  // set it to NULL so the parent class does not segfault
  videoObject->Close ();
  imageObject = NULL;
  tl_errno = TL_NO_ERROR;


  return 0;
}




////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grab (tlImage *image)
{
  // acquire
  imageObject = videoObject->GetImage (WAIT_GRAB);
  color = imageObject->haveColor;

  // initialize images
  if (color) image->init (RGB, width, height);
  else       image->init (GRAY, width, height);

  // proceed to conversion
  if (!color) memcpy (image->pixel, imageObject->Left(), width*height);
  else {
    int           i,j;
    tlPixel       *ipixel = image->pixel;
    unsigned long *iptr   = imageObject->Color();
    for (j=0; j<height; j++) {
      for (i=0; i<width; i++) {
        // copy color pixel data
        *(ipixel++) = (tlPixel)(*(iptr));
        *(ipixel++) = (tlPixel)(*(iptr) >> 8);
        *(ipixel++) = (tlPixel)(*(iptr) >> 16);
        iptr++;
      }
    }
  }


  return 0;
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grab (tlImage *image,
                       tlRect  *rect) 
{
  int x1, x2, y1, y2;


  // shortcuts
  x1 = TL_MAX ((rect->x), 0);
  x2 = TL_MIN (((rect->x)+(rect->width)), width);
  y1 = TL_MAX ((rect->y), 0);
  y2 = TL_MIN (((rect->y)+(rect->height)), height);

  // acquire
  imageObject = videoObject->GetImage (WAIT_GRAB);
  color = imageObject->haveColor;

  // initialize images
  if (color) image->init (RGB, width, height);
  else       image->init (GRAY, width, height);

  // convert monochrome array
  if (!color) {
    int           j;
    tlPixel       *pixel  = image->pixel + y1*width + x1;
    unsigned char *left   = imageObject->Left () + y1*width + x1;
    size_t        length  = x2-x1;
    for (j=y1; j<y2; j++) {
      memcpy (pixel, left, length);
      pixel += width;
      left  += width;
    }
  }

  // convert color array
  else {
    int           i,j;
    tlPixel       *p = image->pixel;
    unsigned long *l = imageObject->Color ();
    for (j=y1; j<y2; j++) {
      tlPixel       *pixel = p + TL_RGB_WIDTH*(j*width+x1);
      unsigned long *left  = l + j*width+x1;
      for (i=x1; i<x2; i++) {
        *(pixel++) = (tlPixel)(*(left));
        *(pixel++) = (tlPixel)(*(left) >> 8);
        *(pixel++) = (tlPixel)(*(left) >> 16);
        left++;
      }
    }
  }


  return 0;
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grab (tlImage *image,
                       tlImage *disparity) 
{
  // acquire
  imageObject = videoObject->GetImage (WAIT_GRAB);
  processObject->CalcStereo (imageObject);
  color = imageObject->haveColor;

  // initialize images
  disparity->init (STEREO, width, height);
  if (color) image->init (RGB, width, height);
  else       image->init (GRAY, width, height);

  // compute disparity scaling factor
  switch (imageObject->dp.ndisp) {
  case 64: disparity->hrShift = 2; break;
  case 32: disparity->hrShift = 1; break;
  default: disparity->hrShift = 0; break;
  }

  // proceed to conversion
  memcpy (disparity->hrpixel, imageObject->Disparity (), width*height*2);
  if (!color) memcpy (image->pixel, imageObject->Left(), width*height);
  else {
    int           i,j;
    tlPixel       *ipixel = image->pixel;
    unsigned long *iptr   = imageObject->Color();
    for (j=0; j<height; j++) {
      for (i=0; i<width; i++) {
        *(ipixel++) = (tlPixel)(*(iptr));
        *(ipixel++) = (tlPixel)(*(iptr) >> 8);
        *(ipixel++) = (tlPixel)(*(iptr) >> 16);
        iptr++;
      }
    }
  }


  return 0;
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grab (tlImage *image,
                       tlImage *disparity,
                       tlRect  *rect) 
{
  grab (image, rect);
  return getDisparity (disparity, rect);
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grabQueued (tlImage *image) 
{
  return grab (image);
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grabQueued (tlImage *image,
                             tlImage *disparity) 
{
  return grab (image, disparity);
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grabQueued (tlImage *image,
                             tlImage *disparity,
                             tlRect  *rect) 
{
  return grab (image, disparity, rect);
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::grabQueued (tlImage *image,
                             tlRect  *rect) 
{
  return grab (image, rect);
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::readConfig (char *filename) 
{
  // read in rectification parameters
  if (!videoObject->ReadParams(filename))  {
    TL_ERRNO_MSG ("tlCameraSVSHead::readConfig()", TL_ERROR_FILE_NOT_FOUND_OR_CORRUPT);
    return -1;
  }

  return 0;
}



////////////////////////////////////////////////////////////////////////

int
tlCameraSVSHead::enableColor (bool val)
{
  videoObject->SetColor (val, imageObject->haveColorRight);
  color = val;


  return 0;
}



////////////////////////////////////////////////////////////////////////

bool
tlCameraSVSHead::enableSTOC (bool val)
{
  if (videoObject->is_proc_capable) {
    if (val ) {
      videoObject->SetProcMode (PROC_MODE_DISPARITY);
      stoc = true;
    }
    else {
      videoObject->SetProcMode (PROC_MODE_OFF);
      stoc = false;
    }
  }
  else stoc = false;


  return stoc;
}


#endif
