/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <math.h>

#include "tlImage.h"
#include "tlColor.h"



///  \file tlImage.h
///  \brief Image object description.

///  \class tlImage
///  \brief Image object, provides access to image data and processing
///
///  Perhaps the most commonly used object in TLIB is the tlImage object. It contains the image
///  data (from its tlPixelArray parent), as well as useful routines such as drawing functions,
///  file read/write, etc. 



///  Constructor.
/// 
///  \note
///  See \ref errors "error management" for details.

tlImage::tlImage () : tlPixelArray ()
{
  return;
}



///  Constructor.
///
///  \param format      image format
///  \param width       image width  [pix]
///  \param height      image height [pix]
/// 
///  \note
///  See \ref errors "error management" for details.

tlImage::tlImage (tl_format format, 
                  int       width, 
                  int       height) : tlPixelArray (format, width, height)
{
  return;
}



///  Constructor.
///
///  \param image       image to copy
/// 
///  \note
///  See \ref errors "error management" for details.

tlImage::tlImage (tlImage * image) : tlPixelArray (image)
{
  return;
}



///  Destructor.
/// 
///  \note
///  See \ref errors "error management" for details.

tlImage::~tlImage ()
{
  tl_errno = TL_NO_ERROR;
}
