/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlImage.h"
#include "tlPoint.h"
#include "tlChain.h"
#include "tlMask.h"
#include "tlObject.h"
#include "tlLine.h"



// globals
static char asciiTable[] = { 0x00, // " "
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x04,	// !
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x00,
                             0x00,
                             0x04,
                             0x0A,	// "
                             0x0A,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,	// #
                             0x0A,
                             0x0A,
                             0x1F,
                             0x0A,
                             0x1F,
                             0x0A,
                             0x0A,
                             0x02,	// $
                             0x02,
                             0x07,
                             0x06,
                             0x03,
                             0x07,
                             0x02,
                             0x02,
                             0x0D,	// %
                             0x0D,
                             0x02,
                             0x02,
                             0x04,
                             0x04,
                             0x0B,
                             0x0B,
                             0x02,	// &
                             0x05,
                             0x05,
                             0x06,
                             0x07,
                             0x0A,
                             0x0B,
                             0x05,
                             0x02,	// '
                             0x02,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x02,	// (
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x02,
                             0x04,	// )
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x04,
                             0x00,	// *
                             0x00,
                             0x0A,
                             0x04,
                             0x0A,
                             0x00,
                             0x00,
                             0x00,
                             0x00,	// +
                             0x00,
                             0x04,
                             0x0E,
                             0x04,
                             0x00,
                             0x00,
                             0x00,
                             0x00,	// ,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x04,
                             0x08,
                             0x00,	// -
                             0x00,
                             0x00,
                             0x00,
                             0x0E,
                             0x00,
                             0x00,
                             0x00,
                             0x00,	// .
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x00,
                             0x04,
                             0x01,	// /
                             0x02,
                             0x02,
                             0x04,
                             0x04,
                             0x08,
                             0x08,
                             0x10,
                             0x06,  // 0
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x06,					
                             0x04,	// 1
                             0x0C,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x0E,
                             0x06,	// 2
                             0x09,
                             0x01,
                             0x01,
                             0x02,
                             0x04,
                             0x08,
                             0x0F,
                             0x06,	// 3
                             0x09,
                             0x01,
                             0x06,
                             0x01,
                             0x01,
                             0x09,
                             0x06,
                             0x02,	// 4
                             0x06,
                             0x0A,
                             0x0A,
                             0x12,
                             0x1F,
                             0x02,
                             0x02,
                             0x0F,	// 5
                             0x08,
                             0x08,
                             0x0E,
                             0x01,
                             0x01,
                             0x01,
                             0x0E,
                             0x06,	// 6
                             0x08,
                             0x08,
                             0x0E,
                             0x09,
                             0x09,
                             0x09,
                             0x06,
                             0x0F,	// 7
                             0x01,
                             0x01,
                             0x02,
                             0x02,
                             0x04,
                             0x04,
                             0x04,
                             0x06,	// 8
                             0x09,
                             0x09,
                             0x06,
                             0x09,
                             0x09,
                             0x09,
                             0x06,
                             0x06,	// 9
                             0x09,
                             0x09,
                             0x09,
                             0x07,
                             0x01,
                             0x09,
                             0x06,
                             0x00,	// :
                             0x00,
                             0x04,
                             0x00,
                             0x00,
                             0x04,
                             0x00,
                             0x00,
                             0x00,	// ;
                             0x00,
                             0x04,
                             0x00,
                             0x00,
                             0x04,
                             0x08,
                             0x00,
                             0x00,	// <
                             0x01,
                             0x02,
                             0x04,
                             0x08,
                             0x04,
                             0x02,
                             0x01,
                             0x00,	// =
                             0x00,
                             0x00,
                             0x0F,
                             0x00,
                             0x0F,
                             0x00,
                             0x00,
                             0x00,	// >
                             0x08,
                             0x04,
                             0x02,
                             0x01,
                             0x02,
                             0x04,
                             0x08,
                             0x06,	// ?
                             0x09,
                             0x01,
                             0x02,
                             0x04,
                             0x04,
                             0x00,
                             0x04,
                             0x06,	// @
                             0x09,
                             0x0B,
                             0x0B,
                             0x08,
                             0x09,
                             0x06,
                             0x00,
                             0x06,	// A
                             0x09,
                             0x09,
                             0x09,
                             0x0F,
                             0x09,
                             0x09,
                             0x09,
                             0x0E,	// B
                             0x09,
                             0x09,
                             0x0E,
                             0x09,
                             0x09,
                             0x09,
                             0x0E,
                             0x06,	// C
                             0x09,
                             0x08,
                             0x08,
                             0x08,
                             0x08,
                             0x09,
                             0x06,
                             0x0E,	// D
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x0E,
                             0x0F,	// E
                             0x08,
                             0x08,
                             0x0E,
                             0x08,
                             0x08,
                             0x08,
                             0x0F,
                             0x0F,	// F
                             0x08,
                             0x08,
                             0x0E,
                             0x08,
                             0x08,
                             0x08,
                             0x08,
                             0x06,	// G
                             0x09,
                             0x08,
                             0x08,
                             0x0B,
                             0x09,
                             0x09,
                             0x06,	// H
                             0x09,
                             0x09,
                             0x09,
                             0x0F,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x07,	// I
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x07,
                             0x07,	// J
                             0x01,
                             0x01,
                             0x01,
                             0x01,
                             0x09,
                             0x09,
                             0x06,
                             0x09,	// K
                             0x09,
                             0x0A,
                             0x0C,
                             0x0C,
                             0x0A,
                             0x09,
                             0x09,
                             0x08,	// L
                             0x08,
                             0x08,
                             0x08,
                             0x08,
                             0x08,
                             0x08,
                             0x0F,
                             0x11,	// M
                             0x1B,
                             0x15,
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x19,	// N
                             0x19,
                             0x15,
                             0x15,
                             0x15,
                             0x15,
                             0x13,
                             0x13,
                             0x0E,	// O
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x0E,
                             0x0E,	// P
                             0x09,
                             0x09,
                             0x0E,
                             0x08,
                             0x08,
                             0x08,
                             0x08,
                             0x0E,	// Q
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x15,
                             0x12,
                             0x0D,
                             0x0E,	// R
                             0x09,
                             0x09,
                             0x0E,
                             0x0C,
                             0x0A,
                             0x09,
                             0x09,
                             0x06,	// S
                             0x09,
                             0x08,
                             0x06,
                             0x01,
                             0x01,
                             0x09,
                             0x06,
                             0x1F,	// T
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x09,	// U
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x06,
                             0x11,	// V
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x0A,
                             0x0A,
                             0x04,
                             0x11,	// W
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x11,
                             0x15,
                             0x0A,
                             0x11,	// X
                             0x0A,
                             0x0A,
                             0x04,
                             0x04,
                             0x0A,
                             0x0A,
                             0x11,
                             0x11,	// Y
                             0x11,
                             0x0A,
                             0x0A,
                             0x04,
                             0x04,
                             0x04,
                             0x0C,
                             0x0F,	// Z
                             0x01,
                             0x02,
                             0x04,
                             0x04,
                             0x08,
                             0x08,
                             0x0F,
                             0x06,	// [
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x04,
                             0x06,
                             0x08,	// "\\"
                             0x08,
                             0x04,
                             0x04,
                             0x02,
                             0x02,
                             0x01,
                             0x01,
                             0x06,	// ]
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x02,
                             0x06,
                             0x0F,	// ^, not implemented
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x09,
                             0x0F,};



///  Draw a pixel on the image with a given color.
/// 
///  \param x   point x location
///  \param y   point y location
///  \param r   desired R color value
///  \param g   desired G color value
///  \param b   desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawPoint (int           x, 
                    int           y, 
                    unsigned char r, 
                    unsigned char g,
                    unsigned char b)
{
  int index;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawPoint()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != RGB) {
    return drawPoint (x, y, (r+g+b)/3);
  }

  // shortcuts
  x = TL_MIN (x, (width - 1));
  x = TL_MAX (x, 0);
  y = TL_MIN (y, (height - 1));
  y = TL_MAX (y, 0);

  // proceed
  index          = TL_RGB_WIDTH * TL_PIXEL (y, x, width);
  pixel[index  ] = r;
  pixel[index+1] = g;
  pixel[index+2] = b;


  return 0;
}



///  Draw a pixel on the single-channel image.
/// 
///  \param x   point x location
///  \param y   point y location
///  \param v   desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawPoint (int           x, 
                    int           y, 
                    unsigned char v)
{
  int index;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawPoint()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::drawPoint()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // shortcuts
  x = TL_MIN (x, (width - 1));
  x = TL_MAX (x, 0);
  y = TL_MIN (y, (height - 1));
  y = TL_MAX (y, 0);

  // proceed
  index        = TL_PIXEL (y, x, width);
  pixel[index] = v;


  return 0;
}



///  Draw a pixel on the image with a given color.
/// 
///  \param point   point location
///  \param r       desired R color value
///  \param g       desired G color value
///  \param b       desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawPoint (tlPoint       *point, 
                    unsigned char r, 
                    unsigned char g,
                    unsigned char b)
{
  return drawPoint ((point->x), (point->y), r, g, b);
}



///  Draw a pixel on the single-channel image.
/// 
///  \param point   point location
///  \param v       desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawPoint (tlPoint       *point, 
                    unsigned char v)
{
  return drawPoint ((point->x), (point->y), v);
}



///  Draw a line on the image with a given color.
/// 
///  \param x1  starting point x location
///  \param y1  starting point y location
///  \param x2  ending point x location
///  \param y2  ending point y location
///  \param r   desired R color value
///  \param g   desired G color value
///  \param b   desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  if the line overshoots the image boundaries, an error is returned

int
tlImage::drawLine (int           x1, 
                   int           y1, 
                   int           x2, 
                   int           y2, 
                   unsigned char r,
                   unsigned char g, 
                   unsigned char b)
{
  register int index;

  int    i,j;
  int	   dx,dy;
  double m;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawLine()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != RGB) {
    return drawLine (x1, y1, x2, y2, (r+g+b)/3);
  }

  // check geometry
  if (
    (x1 >= width) || (x1 < 0) || (y1 >= height) || (y1 < 0) || 
    (x2 >= width) || (x2 < 0) || (y2 >= height) || (y2 < 0)) 
  {
    TL_ERRNO_MSG("tlImage::drawLine()", TL_ERROR_NOT_IN_IMAGE)
    return -1;
  }

  // if line is vertical
  if (x1 == x2) {

    // sort y coord
    if (y2 < y1) {
      int temp;
      temp = y1;
      y1 = y2;
      y2 = temp;
    }
    for (j = y1; j < y2; j++) {
      index = TL_RGB_WIDTH * TL_PIXEL (j, x1, width);
      pixel[index  ] = r;
      pixel[index+1] = g;
      pixel[index+2] = b;
    }
  }

  // non-vertical line
  else {   

    // invert points if necessary
    if (x2 < x1) {
      int temp;
      temp  = x1;
      x1    = x2;
      x2    = temp;
      temp  = y1;
      y1    = y2;
      y2    = temp;
    }

    // shortcuts
    dx = x2 - x1;
    dy = y2 - y1;
    m  = (double) dy / (double) dx;

    // initialization
    i = x1;
    j = y1;

    // if line goes up    
    if (y2 > y1) {
      while (i <= x2) {
        while (j < (int) ((float) y1 + m * (float) (i - x1))) {
          index = TL_RGB_WIDTH * TL_PIXEL (j, i, width);
          pixel[index  ] = r;
          pixel[index+1] = g;
          pixel[index+2] = b;
          j++;
        } 
        index = TL_RGB_WIDTH * TL_PIXEL (j, i, width);
        pixel[index  ] = r;
        pixel[index+1] = g;
        pixel[index+2] = b;
        i++;
      }
    }
   
    // if line goes down
    else {
      while (i <= x2) {
        while (j > (int) ((float) y1 + m * (float) (i - x1))) {
          index = TL_RGB_WIDTH * TL_PIXEL (j, i, width);
          pixel[index  ] = r;
          pixel[index+1] = g;
          pixel[index+2] = b;
          j--;
        } 
        index = TL_RGB_WIDTH * TL_PIXEL (j, i, width);
        pixel[index  ] = r;
        pixel[index+1] = g;
        pixel[index+2] = b;
        i++;
      }
    } 
  }


  return 0;
}



///  Draw a line on the single-channel image.
/// 
///  \param x1  starting point x location
///  \param y1  starting point y location
///  \param x2  ending point x location
///  \param y2  ending point y location
///  \param v   desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  if the line overshoots the image boundaries, an error is returned

int
tlImage::drawLine (int           x1, 
                   int           y1, 
                   int           x2, 
                   int           y2, 
                   unsigned char v)
{
  register int index;

  int    i,j;
  int	   dx,dy;
  double m;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawLine()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::drawLine()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  if (
    (x1 >= width) || (x1 < 0) || (y1 >= height) || (y1 < 0) || 
    (x2 >= width) || (x2 < 0) || (y2 >= height) || (y2 < 0)) 
  {
    TL_ERRNO_MSG("tlImage::drawLine()", TL_ERROR_NOT_IN_IMAGE)
    return -1;
  }

  // if line is vertical
  if (x1 == x2) {

    // sort y coord
    if (y2 < y1) {
      int temp;
      temp = y1;
      y1 = y2;
      y2 = temp;
    }
    for (j = y1; j < y2; j++) {
      index = TL_PIXEL (j, x1, width);
      pixel[index] = v;
    }
  }

  // non-vertical line
  else {   

    // invert points if necessary
    if (x2 < x1) {
      int temp;
      temp  = x1;
      x1    = x2;
      x2    = temp;
      temp  = y1;
      y1    = y2;
      y2    = temp;
    }

    // shortcuts
    dx = x2 - x1;
    dy = y2 - y1;
    m  = (double) dy / (double) dx;

    // initialization
    i = x1;
    j = y1;

    // if line goes up    
    if (y2 > y1) {
      while (i <= x2) {
        while (j < (int) ((float) y1 + m * (float) (i - x1))) {
          index = TL_PIXEL (j, i, width);
          pixel[index] = v;
          j++;
        } 
        index = TL_PIXEL (j, i, width);
        pixel[index] = v;
        i++;
      }
    }
   
    // if line goes down
    else {
      while (i <= x2) {
        while (j > (int) ((float) y1 + m * (float) (i - x1))) {
          index = TL_PIXEL (j, i, width);
          pixel[index] = v;
          j--;
        } 
        index = TL_PIXEL (j, i, width);
        pixel[index] = v;
        i++;
      }
    } 
  }


  return 0;
}



///  Draw a line on the image with a given color.
/// 
///  \param p1 starting point location
///  \param p2 ending point location
///  \param r desired R color value
///  \param g desired G color value
///  \param b desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  if the line overshoots the image boundaries, an error is returned

int
tlImage::drawLine (tlPoint       *p1, 
                   tlPoint       *p2, 
                   unsigned char r,
                   unsigned char g, 
                   unsigned char b)
{
  return drawLine ((p1->x), (p1->y), (p2->x), (p2->y), r, g, b);
}



///  Draw a line on the single-channel image.
/// 
///  \param p1 starting point location
///  \param p2 ending point location
///  \param v desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  if the line overshoots the image boundaries, an error is returned

int
tlImage::drawLine (tlPoint       *p1, 
                   tlPoint       *p2, 
                   unsigned char v)
{
  return drawLine ((p1->x), (p1->y), (p2->x), (p2->y), v);
}



///  Draw a line on the image with a given color.
/// 
///  \param line    line to draw
///  \param r desired R color value
///  \param g desired G color value
///  \param b desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  if the line overshoots the image boundaries, an error is returned

int
tlImage::drawLine (tlLine       *line,
                   unsigned char r,
                   unsigned char g, 
                   unsigned char b)
{
  return drawLine ((line->p1->x), (line->p1->y), (line->p2->x), (line->p2->y), r, g, b);
}



///  Draw a line on the single-channel image.
/// 
///  \param line    line to draw
///  \param v       desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.
/// 
///  \note
///  if the line overshoots the image boundaries, an error is returned

int
tlImage::drawLine (tlLine       *line,
                   unsigned char v)
{
  return drawLine ((line->p1->x), (line->p1->y), (line->p2->x), (line->p2->y), v);
}



///  Draw a cross on the image at the given point, of the given size, 
///  with a given color.
/// 
///  \param x     point X coord
///  \param y     point Y coord
///  \param size  cross segment length
///  \param r     desired R color value
///  \param g     desired G color value
///  \param b     desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (int           x, 
                    int           y, 
                    int           size, 
                    unsigned char r,
                    unsigned char g, 
                    unsigned char b)
{
  int	res;


  tlRect *rect = new tlRect (x - size / 2, y - size / 2, size + (size % 2) - 1,	size + (size % 2) - 1);
  res = this->drawCross (rect, r, g, b);
  delete rect;


  return res;
}



///  Draw a cross on the single-channel image at the given point and of the given size.
/// 
///  \param x     point X coord
///  \param y     point Y coord
///  \param size  cross segment length
///  \param v     desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (int           x, 
                    int           y, 
                    int           size, 
                    unsigned char v)
{
  int	res;


  tlRect *rect = new tlRect (x - size / 2, y - size / 2, size + (size % 2) - 1,	size + (size % 2) - 1);
  res = this->drawCross (rect, v);
  delete rect;


  return res;
}



///  Draw a cross on the image at the given point, of the given size, 
///  with a given color.
/// 
///  \param point the point
///  \param size cross segment length
///  \param r desired R color value
///  \param g desired G color value
///  \param b desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (tlPoint       *point, 
                    int           size, 
                    unsigned char r,
                    unsigned char g, 
                    unsigned char b)
{
  int	res;


  tlRect *rect = new tlRect ((point->x) - size / 2,	(point->y) - size / 2, size + (size % 2) - 1, size + (size % 2) - 1);
  res = this->drawCross (rect, r, g, b);
  delete	rect;


  return res;
}



///  Draw a cross on the image at the given point and of the given size.
/// 
///  \param point   the point
///  \param size    cross segment length
///  \param v       desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (tlPoint       *point, 
                    int           size, 
                    unsigned char v)
{
  int	res;


  tlRect *rect = new tlRect ((point->x) - size / 2,	(point->y) - size / 2, size + (size % 2) - 1, size + (size % 2) - 1);
  res = this->drawCross (rect, v);
  delete	rect;


  return res;
}



///  Draw a cross on the image at every point of the given chain.
/// 
///  \param chain   the chain to draw
///  \param size    cross segment length
///  \param r       desired R color value
///  \param g       desired G color value
///  \param b       desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (tlChain       *chain, 
                    int           size, 
                    unsigned char r,
                    unsigned char g, 
                    unsigned char b)
{
  int	    i;
  int	    limit;
  tlPoint *p;


  limit = chain->getLength ();
  for (i = 0; i < limit; i++) {
    this->drawCross ((p = chain->getPoint (i)), size, r, g, b);
    delete p;
  }


  return 0;
}



///  Draw a cross on the single-channel image at every point of the given chain.
/// 
///  \param chain   the chain to draw
///  \param size    cross segment length
///  \param v       desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (tlChain       *chain, 
                    int           size, 
                    unsigned char v)
{
  int	    i;
  int	    limit;
  tlPoint *p;


  limit = chain->getLength ();
  for (i = 0; i < limit; i++) {
    this->drawCross ((p = chain->getPoint (i)), size, v);
    delete p;
  }


  return 0;
}



///  Draw a cross on the image contained in a rectangle with a given color.
/// 
///  \param rect  the bounding rectangle
///  \param r     desired R color value
///  \param g     desired G color value
///  \param b     desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (tlRect        *rect, 
                    unsigned char r, 
                    unsigned char g,
                    unsigned char b)
{
  register int hor;
  register int vert;

  int	i, j;
  int	x1, x2;
  int	y1,	y2;
  int	dist;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawCross()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != RGB) {
    return drawCross (rect, (r+g+b)/3);
  }

  // check geometry
  x1 = TL_MAX ((rect->x), 0);
  x2 = TL_MIN ((x1 + (rect->width)), width);
  y1 = TL_MAX ((rect->y), 0);
  y2 = TL_MIN ((y1 + (rect->height)), height);

  // horizontal component
  hor = TL_RGB_WIDTH * ((y1 + (y2 - y1) / 2) * width + x1);
  for (i = x1; i < x2; i++) {
    pixel[hor++] = r;
    pixel[hor++] = g;
    pixel[hor++] = b;
  }

  // vertical component
  dist = x1 + (x2 - x1) / 2;
  for (j = y1; j < y2; j++) {
    vert = TL_RGB_WIDTH * TL_PIXEL (j, dist, width);
    pixel[vert++] = r;
    pixel[vert++] = g;
    pixel[vert++] = b;
  }


  return 0;
}



///  Draw a cross on the single-channel image contained in a rectangle.
/// 
///  \param rect  the bounding rectangle
///  \param v     desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawCross (tlRect        *rect, 
                    unsigned char v)
{
  register int hor;
  register int vert;

  int	i, j;
  int	x1, x2;
  int	y1,	y2;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawCross()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::drawCross()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  x1 = TL_MAX ((rect->x), 0);
  x2 = TL_MIN ((x1 + (rect->width)), width);
  y1 = TL_MAX ((rect->y), 0);
  y2 = TL_MIN ((y1 + (rect->height)), height);

  // horizontal component
  hor = ((y1 + (y2 - y1) / 2) * width + x1);
  for (i = x1; i < x2; i++) pixel[hor++] = v;

  // vertical component
  vert = TL_PIXEL (y1, (x1 + (x2 - x1) / 2), width);
  for (j = y1; j < y2; j++) {
    pixel[vert] = v;
    vert += width;
  }


  return 0;
}



///  Draw a rectangle on the image with a given color.
/// 
///  \param rect  the rectangle to draw
///  \param r     desired R color value
///  \param g     desired G color value
///  \param b     desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawRect (tlRect        *rect, 
                   unsigned char r, 
                   unsigned char g,
                   unsigned char b)
{
  register int index1;
  register int index2;

  int	i, j;
  int	x1, x2;
  int	y1,	y2;
  int	dist;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawRect()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != RGB) {
    return drawRect (rect, (r+g+b)/3);
  }

  // check geometry
  x1 = TL_MAX ((rect->x), 0);
  x2 = TL_MIN ((x1 + (rect->width)), width-1);
  y1 = TL_MAX ((rect->y), 0);
  y2 = TL_MIN ((y1 + (rect->height)), height-1);

  // draw top and bottom
  index1 = TL_RGB_WIDTH * TL_PIXEL (y1, x1, width);
  index2 = TL_RGB_WIDTH * TL_PIXEL (y2, x1, width);
  for (i = x1; i < x2; i++) {

    // rectangle top
    pixel[index1++] = r;
    pixel[index1++] = g;
    pixel[index1++] = b;

    // rectangle bottom
    pixel[index2++] = r;
    pixel[index2++] = g;
    pixel[index2++] = b;
  }

  // draw left and right
  dist = TL_RGB_WIDTH * (x2 - x1);
  index1 = TL_RGB_WIDTH * TL_PIXEL (y1, x1, width);
  index2 = index1 + TL_RGB_WIDTH * (x2 - x1);
  for (j = y1; j < y2; j++) {

    // one pixel to the left...
    pixel[index1++] = r;
    pixel[index1++] = g;
    pixel[index1++] = b;

    // ... and one pixel to the right !
    pixel[index2++] = r;
    pixel[index2++] = g;
    pixel[index2++] = b;

    // update
    index1 += TL_RGB_WIDTH*(width-1);
    index2 = index1 + dist;
  }


  return 0;
}



///  Draw a rectangle on the single-channel image.
/// 
///  \param rect  the rectangle to draw
///  \param v     desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawRect (tlRect        *rect, 
                   unsigned char v)
{
  register int index1;
  register int index2;

  int	i, j;
  int	x1, x2;
  int	y1,	y2;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawRect()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::drawRect()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  x1 = TL_MAX ((rect->x), 0);
  x2 = TL_MIN ((x1 + (rect->width)), width-1);
  y1 = TL_MAX ((rect->y), 0);
  y2 = TL_MIN ((y1 + (rect->height)), height-1);

  // draw top and bottom
  index1 = TL_PIXEL (y1, x1, width);
  index2 = TL_PIXEL (y2, x1, width);
  for (i = x1; i < x2; i++) {
    pixel[index1++] = v;
    pixel[index2++] = v;
  }

  // draw left and right
  index1 = TL_PIXEL (y1, x1, width);
  index2 = index1 + (x2 - x1);
  for (j = y1; j < y2; j++) {
    pixel[index1] = v;
    pixel[index2] = v;
    index1 += width;
    index2 += width;
  }


  return 0;
}



///  Draw a mask on the image at a given location 
///  and with a given color.
/// 
///  \param mask  the mask to draw
///  \param x     the X coordinate (UL corner) of the mask on the image
///  \param y     the Y coordinate (UL corner) of the mask on the image
///  \param r     desired R color value
///  \param g     desired G color value
///  \param b     desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawMask (tlMask        *mask, 
                   int           x, 
                   int           y, 
                   unsigned char r,
                   unsigned char g, 
                   unsigned char b)
{
  register int index;
  register int mask_index;

  int	    i, j;
  int	    jj;
  int	    x2,	y2;
  int	    mask_width;
  tlPixel *mask_src;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawMask()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != RGB) {
    return drawMask (mask, x, y, (r+g+b)/3);
  }

  // check geometry
  if ((x < 0) || (x >= width) || (y < 0) || (y >= height)) {
    TL_ERRNO_MSG ("tlImage::drawMask()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

  // shortcuts
  mask_src   = mask->pixel;
  mask_width = mask->width;
  x2         = TL_MIN ((x + (mask->width)), (width - 1));
  y2         = TL_MIN ((y + (mask->height)), (height - 1));

  // browse mask
  for (j = y, jj = 0; j < y2; j++, jj++) {
    index = TL_RGB_WIDTH * TL_PIXEL (j, x, width);
    mask_index = TL_PIXEL (jj, 0, mask_width);
    for (i = x; i < x2; i++) {
      if (mask_src[mask_index]) {
        pixel[index  ] = r;
        pixel[index+1] = g;
        pixel[index+2] = b;
      }
      index += TL_RGB_WIDTH;
      mask_index += 1;
    }
  }


  return 0;
}



///  Draw a mask on the single-channel image at a given location.
/// 
///  \param mask  the mask to draw
///  \param x     the X coordinate (UL corner) of the mask on the image
///  \param y     the Y coordinate (UL corner) of the mask on the image
///  \param v     desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawMask (tlMask        *mask, 
                   int           x, 
                   int           y, 
                   unsigned char v)
{
  register int index;
  register int mask_index;

  int	    i, j;
  int	    jj;
  int	    x2,	y2;
  int	    mask_width;
  tlPixel *mask_src;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawMask()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::drawMask()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  if ((x < 0) || (x >= width) || (y < 0) || (y >= height)) {
    TL_ERRNO_MSG ("tlImage::drawMask()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

  // shortcuts
  mask_src   = mask->pixel;
  mask_width = mask->width;
  x2         = TL_MIN ((x + (mask->width)), (width - 1));
  y2         = TL_MIN ((y + (mask->height)), (height - 1));

  // browse mask
  for (j = y, jj = 0; j < y2; j++, jj++) {
    index = TL_PIXEL (j, x, width);
    mask_index = TL_PIXEL (jj, 0, mask_width);
    for (i = x; i < x2; i++) {
      if (mask_src[mask_index]) pixel[index] = v;
      index      += 1;
      mask_index += 1;
    }
  }


  return 0;
}



///  Draw a mask on the image at its stored location 
///  and with a given color.
/// 
///  \param mask the mask to draw
///  \param r desired R color value
///  \param g desired G color value
///  \param b desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawMask (tlMask        *mask, 
                   unsigned char r, 
                   unsigned char g,
                   unsigned char b)
{
  return drawMask (mask, (mask->x), (mask->y), r, g, b);
}



///  Draw a mask on the single-channel image at its stored location.
/// 
///  \param mask  the mask to draw
///  \param v     desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawMask (tlMask        *mask, 
                   unsigned char v)
{
  return drawMask (mask, (mask->x), (mask->y), v);
}



///  Draw a chain representation on the image.
/// 
///  \param chain   the chain to draw
///  \param r       desired R color value
///  \param g       desired G color value
///  \param b       desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawChain (tlChain       *chain, 
                    unsigned char r, 
                    unsigned char g,
                    unsigned char b)
{
  int	    i;
  int	    limit;
  tlPoint *p;


  limit = chain->getLength ();
  for (i = 0; i < limit; i++) {
    this->drawPoint ((p = chain->getPoint (i)), r, g, b);
    delete p;
  }


  return 0;
}



///  Draw a chain representation on the single-channel image.
/// 
///  \param chain   the chain to draw
///  \param v       desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawChain (tlChain       *chain, 
                    unsigned char v)
{
  int	    i;
  int	    limit;
  tlPoint *p;


  limit = chain->getLength ();
  for (i = 0; i < limit; i++) {
    this->drawPoint ((p = chain->getPoint (i)), v);
    delete p;
  }


  return 0;
}



///  Draw an object representation on the image.
/// 
///  \param object the object to draw
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawObject (tlObject * object)
{
  return drawObject (object, object->x, object->y);
}




///  Draw an object representation on the image at a given location.
/// 
///  \param object  the object to draw
///  \param x       object x location
///  \param y       object y location
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawObject (tlObject *object, 
                     int      x, 
                     int      y)
{
  register int index;
  register int object_index;
  register int mask_index;

  int	    i, j, k;
  int	    jj;
  int	    x2, y2;
  int	    object_width;
  tlPixel *object_src;
  tlPixel *mask_src;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawObject()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != object->format) {
    TL_ERRNO_MSG ("tlImage::drawObject()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  if ((x < 0) || (x > (width - 1)) || (y < 0) || (y > (height - 1))) {
    TL_ERRNO_MSG ("tlImage::drawObject()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

  // shortcuts
  object_src   = object->pixel;
  object_width = object->width;
  mask_src     = object->shape->pixel;
  x2           = TL_MIN ((x + (object->width)), width);
  y2           = TL_MIN ((y + (object->height)), height);

  // browse mask
  for (j = y, jj = 0; j < y2; j++, jj++) {
    index        = pixelWidth * TL_PIXEL (j, x, width);
    mask_index   = jj * object_width;
    object_index = mask_index * pixelWidth;
    for (i = x; i < x2; i++) {
      if (mask_src[mask_index]) {
        for (k = 0; k < pixelWidth; k++)
          pixel[index + k] = object_src[object_index + k];
      }
      index        += pixelWidth;
      mask_index   += 1;
      object_index += pixelWidth;
    }
  }


  return 0;
}



///  Draw an image on the image at a given location.
/// 
///  \param image   the image to draw
///  \param x       object x location
///  \param y       object y location
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawImage (tlImage *image, 
                    int     x, 
                    int     y)
{
  register tlPixel *src;
  register tlPixel *dest;

  int	    i, j, k;
  int	    jj;
  int	    x2, y2;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawImage()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != image->format) {
    TL_ERRNO_MSG ("tlImage::drawImage()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  if ((x < 0) || (x > (width - 1)) || (y < 0) || (y > (height - 1))) {
    TL_ERRNO_MSG ("tlImage::drawImage()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

  // shortcuts
  dest        = pixel;
  x2          = TL_MIN ((x + (image->width  )), width);
  y2          = TL_MIN ((y + (image->height)), height);

  // browse mask
  src = image->pixel;
  for (j = y, jj = 0; j < y2; j++, jj++) {
    dest = pixel + pixelWidth * TL_PIXEL (j, x, width);
    for (i = x; i < x2; i++) {
      for (k = 0; k < pixelWidth; k++)
        *(dest++) = *(src++);
    }
  }


  return 0;
}



///  Fill up a rectangle on the image with a desired color.
/// 
///  \param rect  the rectangle to draw
///  \param r     desired R color value
///  \param g     desired G color value
///  \param b     desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::fillRect (tlRect        *rect, 
                   unsigned char r, 
                   unsigned char g,
                   unsigned char b)
{
  register tlPixel *src;

  int	i,  j;
  int	x1, x2;
  int	y1, y2;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::fillRect()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (format != RGB) {
    return fillRect (rect, (r+g+b)/3);
  }

  // shortcuts
  x1 = TL_MAX ((rect->x), 0);
  x2 = TL_MIN ((x1 + (rect->width)), width);
  y1 = TL_MAX ((rect->y), 0);
  y2 = TL_MIN ((y1 + (rect->height)), height);

  // draw rect
  for (j = y1; j < y2; j++) {
    src = pixel + TL_RGB_WIDTH * TL_PIXEL (j, x1, width);
    for (i = x1; i < x2; i++) {
      *(src++) = r;
      *(src++)  = g;
      *(src++)  = b;
    }
  }


  return 0;
}



///  Fill up a rectangle on the single-channel image.
/// 
///  \param rect  the rectangle to draw
///  \param v     desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::fillRect (tlRect        *rect, 
                   unsigned char v)
{
  register tlPixel *src;

  int	i,  j;
  int	x1, x2;
  int	y1, y2;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::fillRect()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::fillRect()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // shortcuts
  x1 = TL_MAX ((rect->x), 0);
  x2 = TL_MIN ((x1 + (rect->width)), width);
  y1 = TL_MAX ((rect->y), 0);
  y2 = TL_MIN ((y1 + (rect->height)), height);

  // draw rect
  for (j = y1; j < y2; j++) {
    src = pixel + TL_PIXEL (j, x1, width);
    for (i = x1; i < x2; i++) *(src++) = v;
  }


  return 0;
}



///  Fill an object on the image with a given color
///  at a desired location.
/// 
///  \param object  the object to draw
///  \param x       desired object x location
///  \param y       desired object y location
///  \param r       desired R color value
///  \param g       desired G color value
///  \param b       desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::fillObject (tlObject      *object, 
                     int           x, 
                     int           y, 
                     unsigned char r,
                     unsigned char g, 
                     unsigned char b)
{
  register int index;
  register int mask_index;

  int	    i, j;
  int	    jj;
  int	    x2, y2;
  int	    object_width;
  tlPixel *mask_src;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::fillObject()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!object) {
    TL_ERRNO_MSG ("tlImage::fillObject()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // check format
  if (format != RGB) {
    return fillObject (object, x, y, (r+g+b)/3);
  }

  // check geometry
  if ((x < 0) || (x > (width - 1)) || (y < 0) || (y > (height - 1))) {
    TL_ERRNO_MSG ("tlImage::fillObject()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

  // shortcuts
  object_width = object->width;
  mask_src     = object->shape->pixel;
  x2           = TL_MIN ((x + (object->width)), width);
  y2           = TL_MIN ((y + (object->height)), height);

  // browse mask
  for (j = y, jj = 0; j < y2; j++, jj++) {
    index        = pixelWidth * TL_PIXEL (j, x, width);
    mask_index   = jj * object_width;
    for (i = x; i < x2; i++) {
      if (mask_src[mask_index] != 0) {
        pixel[index  ] = r;
        pixel[index+1] = g;
        pixel[index+2] = b;
      }
      index        += pixelWidth;
      mask_index   += 1;
    }
  }


  return 0;
}



///  Fill an object on the single-channel image at a desired location.
/// 
///  \param object  the object to draw
///  \param x       desired object x location
///  \param y       desired object y location
///  \param v       desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::fillObject (tlObject      *object, 
                     int           x, 
                     int           y, 
                     unsigned char v)
{
  register int index;
  register int mask_index;

  int	    i, j;
  int	    jj;
  int	    x2, y2;
  int	    object_width;
  tlPixel *mask_src;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::fillObject()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!object) {
    TL_ERRNO_MSG ("tlImage::fillObject()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::fillObject()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  if ((x < 0) || (x > (width - 1)) || (y < 0) || (y > (height - 1))) {
    TL_ERRNO_MSG ("tlImage::fillObject()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

  // shortcuts
  object_width = object->width;
  mask_src     = object->shape->pixel;
  x2           = TL_MIN ((x + (object->width)), width);
  y2           = TL_MIN ((y + (object->height)), height);

  // browse mask
  for (j = y, jj = 0; j < y2; j++, jj++) {
    index        = TL_PIXEL (j, x, width);
    mask_index   = jj * object_width;
    for (i = x; i < x2; i++) {
      if (mask_src[mask_index] != 0) {
        pixel[index] = v;
      }
      index      += 1;
      mask_index += 1;
    }
  }


  return 0;
}



///  Fill an object on the image with a given color
///  at the object stored location.
/// 
///  \param object  the object to draw
///  \param r       desired R color value
///  \param g       desired G color value
///  \param b       desired B color value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::fillObject (tlObject      *object, 
                     unsigned char r, 
                     unsigned char g,
                     unsigned char b)
{
  return fillObject (object, (object->x), (object->y), r, g, b);
}



///  Fill an object on the image with a given color
///  at the object stored location.
/// 
///  \param object  the object to draw
///  \param v       desired intensity value
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::fillObject (tlObject      *object, 
                     unsigned char v)
{
  return fillObject (object, (object->x), (object->y), v);
}



///  Draw the requested text on the image.
/// 
///  \param text      the text to draw
///  \param p         text location
///  \param r         desired R color value
///  \param g         desired G color value
///  \param b         desired B color value
///  \param blackbg   true for black background, false for transparent
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawText (char          *text, 
                   tlPoint       *p,
                   unsigned char r,
                   unsigned char g,
                   unsigned char b,
                   bool          blackbg)
{
  return drawText (text, p->x, p->y, r, g, b, blackbg);
}



///  Draw the requested text on the single-channel image.
/// 
///  \param text      the text to draw
///  \param p         text location
///  \param v         desired intensity value
///  \param blackbg   true for black background, false for transparent
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawText (char          *text, 
                   tlPoint       *p,
                   unsigned char v,
                   bool          blackbg)
{
  return drawText (text, p->x, p->y, v, blackbg);
}



///  Draw the requested text on the image.
/// 
///  \param text      the text to draw
///  \param x         text X location
///  \param y         text Y location
///  \param r         desired R color value
///  \param g         desired G color value
///  \param b         desired B color value
///  \param blackbg   true for black background, false for transparent
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawText (char          *text, 
                   int           x,
                   int           y,
                   unsigned char r,
                   unsigned char g,
                   unsigned char b,
                   bool          blackbg)
{
  int     i, length;
  int     offset, lineOffset;
	char    pixelLine;
	tlPixel *lp;
  tlPixel *sp;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawText()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!text) {
    TL_ERRNO_MSG ("tlImage::drawText()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // check format
  if (format != RGB) {
    return drawText (text, x, y, (r+g+b)/3, blackbg);
  }

  // check geometry
  if ((x < 0) || (x > (width - 1)) || (y < 0) || (y > (height - 1))) {
    TL_ERRNO_MSG ("tlImage::drawText()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

	// shortcuts
	sp         = pixel;
  offset     = (pixelWidth * y * width) + (pixelWidth * x);
  lineOffset = pixelWidth * width;

  // determine max number of characters that fit
  length = TL_MIN((int)strlen(text), (width-x)/5);

	for (int line=0; line<8; line++) {
		if ((y+line)>=0 && (y+line)<height) {
			lp = sp + offset + line * lineOffset;
			for (i=0; i<length; i++) {
				char c = text[i];

        // capitalize text
				if(c <= 0x7A && c >= 0x61) c=c-0x20;

        // display empty square for unknown characters
        else if(c < 0x20 || c > 0x5D)	c='^';

        // shift by 0x20 to get asciiTable index
				c = c-0x20;

        // get bitwise pixel line
				pixelLine = asciiTable[(c*8)+line];

        // start on fourth col
				pixelLine = pixelLine << 3;

        // draw corresponding pixels
        for (int col=3; col<8; col++) {
          if(pixelLine & 128) {
            *(lp  ) = r;
            *(lp+1) = g;
            *(lp+2) = b;
          }
          else if (blackbg) {
            *(lp  ) = 0;
            *(lp+1) = 0;
            *(lp+2) = 0;
          }
          lp += 3;
          pixelLine = pixelLine << 1;
        }
      }
    }
  }


  return 0;
}



///  Draw the requested text on the single-channel image.
/// 
///  \param text      the text to draw
///  \param x         text X location
///  \param y         text Y location
///  \param v         desired intensity value
///  \param blackbg   true for black background, false for transparent
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlImage::drawText (char          *text, 
                   int           x,
                   int           y,
                   unsigned char v,
                   bool          blackbg)
{
  int     i, length;
  int     offset;
	char    pixelLine;
	tlPixel *lp;
  tlPixel *sp;


  // check
  if (hr) {
    TL_ERRNO_MSG ("tlImage::drawText()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!text) {
    TL_ERRNO_MSG ("tlImage::drawText()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // check format
  if (pixelWidth != 1) {
    TL_ERRNO_MSG ("tlImage::drawText()", TL_ERROR_INVALID_FORMAT);
    return -1;
  }

  // check geometry
  if ((x < 0) || (x > (width - 1)) || (y < 0) || (y > (height - 1))) {
    TL_ERRNO_MSG ("tlImage::drawText()", TL_ERROR_INDEX_OUT_OF_BOUNDS);
    return -1;
  }

	// shortcuts
	sp     = pixel;
  offset = (y * width) + x;

  // determine max number of characters that fit
  length = TL_MIN((int)strlen(text), (width-x)/5);

	for (int line=0; line<8; line++) {
		if ((y+line)>=0 && (y+line)<height) {
			lp = sp + offset + line * width;
			for (i=0; i<length; i++) {
				char c = text[i];

        // capitalize text
				if(c <= 0x7A && c >= 0x61) c=c-0x20;

        // display empty square for unknown characters
        else if(c < 0x20 || c > 0x5D)	c='^';

        // shift by 0x20 to get asciiTable index
				c = c-0x20;

        // get bitwise pixel line
				pixelLine = asciiTable[(c*8)+line];

        // start on fourth col
				pixelLine = pixelLine << 3;

        // draw corresponding pixels
        for (int col=3; col<8; col++) {
          if (pixelLine & 128) {
            *(lp) = v;
          }
          else if (blackbg) {
            *(lp) = 0;
          }
          lp ++;
          pixelLine = pixelLine << 1;
        }
      }
    }
  }


  return 0;
}
