/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlPixelArray.h"
#include "tlHist.h"
#include "tlMask.h"



///  Build a histogram of a given channel for a portion of a pixel array.
///
///  \param mask pixel array mask (do not consider inactive pixels)
///  \param rect portion of the pixel array to consider
///  \param channel channel to extract
///  \param hist \b [out] the resulting histogram 
///
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlPixelArray::buildHist (tlHist *hist, 
                         tlMask *mask, 
                         tlRect *rect,
                         int    channel)
{
  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  if (!hr) return hist->build (pixel,   width, height, pixelWidth, mask->pixel, channel, rect);
  else     return hist->build (hrpixel, width, height, pixelWidth, mask->pixel, channel, rect);
}



/// \overload

int
tlPixelArray::buildHist (tlHist *hist, 
                         int    channel)
{
  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // proceed
  if (!hr) return hist->build (pixel,   width, height, pixelWidth, NULL, channel, this);
  else     return hist->build (hrpixel, width, height, pixelWidth, NULL, channel, this);
}



/// \overload

int
tlPixelArray::buildHist (tlHist *hist, 
                         tlRect *rect, 
                         int    channel)
{
  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // proceed
  if (!hr) return hist->build (pixel,   width, height, pixelWidth, NULL, channel, rect);
  else     return hist->build (hrpixel, width, height, pixelWidth, NULL, channel, rect);
}



/// \overload

int
tlPixelArray::buildHist (tlHist *hist, 
                         tlMask *mask, 
                         int    channel)
{
  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  if (!hr) return hist->build (pixel,   width, height, pixelWidth, mask->pixel, channel, this);
  else     return hist->build (hrpixel, width, height, pixelWidth, mask->pixel, channel, this);
}
