/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlPixelArray.h"
#include "tlHist2D.h"
#include "tlMask.h"



///  Build a 2D histogram of two given channels from a portion of a pixel array.
///
///  \param mask      pixel array mask (do not consider inactive pixels)
///  \param rect      portion of the pixel array to consider
///  \param xchannel  channel to extract
///  \param ychannel  channel to extract
///  \param hist      \b [out] a valid pointer to the resulting histogram 
///
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlPixelArray::buildHist (tlHist2D *hist, 
                         tlMask   *mask, 
                         tlRect   *rect,
                         int      xchannel, 
                         int      ychannel)
{
  // only for single-byte image
  if (hr) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist2D()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  return hist->build (pixel, width, height, pixelWidth, mask->pixel, xchannel, ychannel, rect);
}



///  \overload

int
tlPixelArray::buildHist (tlHist2D *hist, 
                         int      xchannel, 
                         int      ychannel)
{
  int	res;


  // only for single-byte image
  if (hr) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist2D()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // proceed
  res =	hist->build (pixel, width, height, pixelWidth, NULL, xchannel, ychannel,this);


  return res;
}



///  \overload

int
tlPixelArray::buildHist (tlHist2D *hist, 
                         tlRect   *rect, 
                         int      xchannel,
                         int      ychannel)
{
  int	res;


  // only for single-byte image
  if (hr) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist2D()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  // proceed
  res =	hist->build (pixel, width, height, pixelWidth, NULL, xchannel, ychannel, rect);


  return res;
}



/// \overload

int
tlPixelArray::buildHist (tlHist2D *hist, 
                         tlMask   *mask, 
                         int      xchannel,
                         int      ychannel)
{
  // only for single-byte image
  if (hr) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_HIGHRES_INCOMPATIBLE);
    return -1;
  }

  // check
  if (!hist) {
    TL_ERRNO_MSG ("tlPixelArray::buildHist()", TL_ERROR_NULL_POINTER);
    return -1;
  }

  return hist->build (pixel, width, height, pixelWidth, mask->pixel, xchannel, ychannel, this);
}
