/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlSize.h"



///  \file tlSize.h
///  \brief Geometric size object description.

///  \class tlSize
///  \brief Size object


///  \var int tlSize::width
///  width of the object inheriting from tlSize.

///  \var int tlSize::height
///  height of the object inheriting from tlSize.

///  \var int tlSize::depth
///  disparity range size of the object inheriting from tlSize.



///  \fn tlSize::~tlSize
///  Destructor.



///  Constructor.
///
///  \note
///  See \ref errors "error management" for details.

tlSize::tlSize ()
{
  width = 0;
  height = 0;
  depth = 255;

  tl_errno = TL_NO_ERROR;
}



///  Constructor.
///
///  \param width         object width
///  \param height        object height
///  \param depth				 object disparity range size
///
///  \note
///  See \ref errors "error management" for details.

tlSize::tlSize (int width, 
                int height, 
                int depth)
{
  this->width = width;
  this->height = height;
  this->depth = depth;

  tl_errno = TL_NO_ERROR;
}



///  Destructor.
///
///  \note
///  See \ref errors "error management" for details.

tlSize::~tlSize ()
{
  tl_errno = TL_NO_ERROR;
}



///  Set the size object parameters.
/// 
///  \param width   object width
///  \param height  object height
///  \param depth   object disparity range size
/// 
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tlSize::setSize (int width, 
                 int height, 
                 int depth)
{
  this->width  = width;
  this->height = height;
  this->depth  = depth;

  return 0;
}
