/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <math.h>
#include <memory.h>

#ifdef LINUX
#include <stdlib.h>
#endif

#include "tlTransforms.h"



///  \file tlTransforms.h
///  \brief Internal image transform operators.
///  \internal



///  Rotate a pixel array clockwise, put result in another array.
///  
///  \param src           image pixel array
///  \param width         image width
///  \param height        image height
///  \param pixel_width   pixel width
///  \param dest          \b [out] destination pixel array 
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tl_rotate_right (tlPixel *src, 
                 int     width, 
                 int     height, 
                 int     pixel_width,
                 tlPixel *dest)
{
  register tlPixel *in;
  register tlPixel *out;

  int i, j, k;


  // shortcuts
  in = src;

	// proceed
	for (j=0; j<height; j++) {
    out = dest + pixel_width*((height-j)-1);
    for (i=0; i<width; i++) {
      for (k=0; k<pixel_width; k++) {
        *(out++) = *(in++);
      }
      out += pixel_width*(height-1);
		}
	}
	

	return 0;
}



///  Rotate a pixel array counter-clockwise, put result in another array.
///  
///  \param src           image pixel array
///  \param width         image width
///  \param height        image height
///  \param pixel_width   pixel width
///  \param dest          \b [out] destination pixel array 
///  
///  \return
///  0 on success, -1 otherwise.\n
///  See \ref errors "error management" for details.

int
tl_rotate_left (tlPixel *src, 
                int     width, 
                int     height, 
                int     pixel_width,
                tlPixel *dest)
{
  register tlPixel *in;
  register tlPixel *out;

  int i, j, k;


  // shortcuts
  in = src;

	// proceed
	for (j=0; j<height; j++) {
    out = dest + pixel_width*(height*(width-1)+j);
    for (i=0; i<width; i++) {
      for (k=0; k<pixel_width; k++) {
        *(out++) = *(in++);
      }
      out -= pixel_width*(height+1);
		}
	}
	

	return 0;
}

